import { Strategy } from "./strategy";
import { Options } from "./options";
/**
 * B-Heap implementation. It's like a binary heap, but with fewer page faults.
 *
 * This is transcribed from http://phk.freebsd.dk/B-Heap/binheap.c. We use
 * "algo 3", as the others are proven slower.
 *
 * Why a B-Heap and not a binary heap? A B-Heap improves memory locality. Since
 * we often deal with subtrees, we want the data in subtrees to be close
 * together. A binary tree is terrible at this.
 */
export declare class BHeapStrategy implements Strategy {
    readonly options: Options;
    constructor(options: Options);
    get comparator(): (a: number, b: number) => number;
    pageSize: number;
    length: number;
    private _shift;
    private _emptyMemoryPageTemplate;
    private _memory;
    private _mask;
    queue(value: any): void;
    dequeue(): any;
    peek(): any;
    clear(): void;
    private _write;
    private _read;
    private _bubbleUp;
    private _bubbleDown;
}
